package com.xhly.manageapp

import DownLoadUtils
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.provider.Settings
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.hjq.toast.Toaster
import com.xhkjedu.manageapp.R
import com.xhkjedu.manageapp.databinding.ItemAppBinding
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.eventbus.UIEvent
import com.xhly.corelib.utils.AppUtils
import com.xhly.corelib.utils.GsonUtils
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.ModelNameUtils
import com.xhly.corelib.utils.NetworkUtils
import com.xhly.corelib.utils.SharedPreferencesUtils
import com.xhly.corelib.utils.SystemUtil
import com.xhly.manageapp.bean.scheme.AutoAppBean
import com.xhly.manageapp.bean.user.UserBean
import com.xhly.manageapp.utils.InstallUtils
import com.xhly.manageapp.utils.StrategyUtils
import java.util.Calendar

class AppAdapter(var context: Context, var data: ArrayList<AppInfo>) :
    RecyclerView.Adapter<AppAdapter.AppViewHolder>() {
     var changePwdListener:ChangePwdListener?=null
    //用来判断1分钟内的连续策略请求
     var lastTimeLong=0L
     //用来判断1s内的连点
     var lastClickLong=0L
     var userBean:UserBean?=null
    val spUtils by lazy { SharedPreferencesUtils.getInstance(context) }
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AppViewHolder {
        val inflate = ItemAppBinding.inflate(LayoutInflater.from(context), parent, false)
        return AppViewHolder(inflate)
    }

    override fun getItemCount(): Int {
        return data.size
    }




    override fun onBindViewHolder(holder: AppViewHolder, position: Int) {
        val appInfo = data[position]
        holder.mBinding.itemCv.setImageDrawable(appInfo.ico)
        holder.mBinding.itemCv.setText(appInfo.name)
        holder.mBinding.itemCv.setOnClickListener {
             if (System.currentTimeMillis()-lastClickLong<1000){
                 return@setOnClickListener
             }
             lastClickLong=System.currentTimeMillis()
            if (appInfo.packageName.equals(Const.CUSTOMSPEEDAPP)){
                changePwdListener?.let {
                    it.clearMemory(context,data)
                }
            }else if (appInfo.packageName.equals(Const.CUSTOMCLEARAPP)){
                changePwdListener?.let {
                    it.clearAppData(context)
                }
            }else if (appInfo.packageName.equals(Const.CUSTOMROTATIONAPP)){
                val rotationAllowed = SystemUtil.isRotationAllowed(context)
                if (ModelNameUtils.IS_LianxX505f()){
                    ManageApplication.disallowSetAutoRotation(rotationAllowed)
                }else  if (ModelNameUtils.IS_LianxX6e6f()||ModelNameUtils.IS_LianxTB223FC()){
                    ManageApplication.disallowSetAutoRotation(!rotationAllowed)
                }
                val tips=if (SystemUtil.isRotationAllowed(context)){
                    context.getString(R.string.closerotation)
                }else{
                    context.getString(R.string.openrotation)
                }
                Toaster.showLong(tips)
            }else if (appInfo.packageName.equals(Const.CUSTOMCHANGEPSD)){
                changePwdListener?.let {
                    it.changeCode()
                }
            }else if (appInfo.packageName.equals(Const.UPDATESTRATEGYPKGNAME)){
               if (NetworkUtils.isAvailable(context)){
                   if (lastTimeLong+20*1000<System.currentTimeMillis()){
                       lastTimeLong=System.currentTimeMillis()
                       UIEvent(Const.ADAPTER2004).post()
                   }else{
                       Toaster.show(context.getString(R.string.strategyupdatetips))
                   }
               }else{
                   Toaster.show(context.getString(R.string.checknettips))
               }
            }else if (appInfo.packageName.equals(Const.WIFISEETTINGPKG)){
                val intent = Intent(Settings.ACTION_WIFI_SETTINGS)
                intent.putExtra("full_screen",true)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                context.startActivity(intent)
            }else if (appInfo.packageName.equals(Const.STOREAPPPKG)){
                //跳转应用商店
                if (!AppUtils.isAppInstalled(context,appInfo.packageName)){
                    Toaster.show(context.getString(R.string.startapptips))
                    return@setOnClickListener
                }
                var destroyFlag=false
                var useFlag=true
                try {
                    val appTimeStrategy = StrategyUtils.getAppTimeStrategy(context, appInfo.packageName)
                    if (appTimeStrategy){
                        val intent: Intent? =
                            context.getPackageManager().getLaunchIntentForPackage(Const.STOREAPPPKG)
                        if (intent != null) {
                            Const.startAppStoreFlag=true
                            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                            intent.putExtra("isInstalling",InstallUtils.isInstalling)
                            context.startActivity(intent)
                            destroyFlag=true
                        }else{
                            destroyFlag=false
                        }
                    }else{
                        useFlag=false
                        Toaster.showLong(context.getString(R.string.apptimestoptips))
                    }
                }catch (e:Exception){
                    destroyFlag=false
                    LogShow("失败原因"+e.toString())
                }finally {
                    if (destroyFlag){
                        DownLoadUtils.destroyDownLoad()
                        InstallUtils.destroyInstallUtils()
                    }else{
                        if (useFlag){
                            //提示了不再可用时间则不再提示未安装
                            Toaster.show(context.getString(R.string.startapptips))
                        }
                    }
                }
            } else{
                try {
                    if (!AppUtils.isAppInstalled(context,appInfo.packageName)){
                        Toaster.show("应用正在安装，请稍后")
                        return@setOnClickListener
                    }
                    val instance = Calendar.getInstance()
                    instance.set(Calendar.HOUR_OF_DAY, 11)
                    instance.set(Calendar.MINUTE, 0)
                    /* context.startActivity(appInfo.intent)*/
                    val appTimeStrategy = StrategyUtils.getAppTimeStrategy(context, appInfo.packageName)
                    if (appTimeStrategy){
                        //然后根据自动拉起和单点登录判断打开时是否传递登录数据
                        if (userBean!=null){
                            userBean?.let {
                                if (it.sso==2){
                                    //单点登录则传递数据，否则不传递,单点登录同样有两种情况，一种是拉起，一种是不拉起，拉起的则指定页面跳转，否则不指定
                                    val autoAppBean = spUtils.getFromJson(
                                        Const.AUTOAPPKEY,
                                        AutoAppBean().javaClass
                                    ) as? AutoAppBean
                                    if (autoAppBean!=null&&autoAppBean.appPackage!=null&&autoAppBean.authUrl!=null&&autoAppBean.appPackage == appInfo.packageName){
                                        val launchIntent = Intent()
                                        launchIntent.setComponent(
                                            ComponentName(
                                                autoAppBean.appPackage!!,
                                                autoAppBean.authUrl!!
                                            )
                                        )
                                        launchIntent.putExtra("authType", autoAppBean.authType)
                                        userBean?.let {
                                            val parseClassToJson = GsonUtils.parseClassToJson(it)
                                            launchIntent.putExtra("authData", parseClassToJson)
                                        }
                                        context.startActivity(launchIntent)
                                    }else{
                                        //单点不拉起，则传递数据，但是不指定页面专挑
                                        val intent = appInfo.intent
                                        intent.putExtra("authType", userBean?.authType)
                                        userBean?.let {
                                            val parseClassToJson = GsonUtils.parseClassToJson(it)
                                            intent.putExtra("authData", parseClassToJson)
                                        }
                                        context.startActivity(intent)
                                    }
                                }else{
                                    //非单点登录还走之前的跳转
                                    context.startActivity(appInfo.intent)
                                }
                            }
                        }else{
                            context.startActivity(appInfo.intent)
                        }
                    }else{
                        Toaster.showLong(context.getString(R.string.apptimestoptips))
                    }
                }catch (e:Exception){
                    LogShow("应用跳转问题是"+e.toString())
                }   
            }
        }
    }

    class AppViewHolder(binding: ItemAppBinding) : RecyclerView.ViewHolder(binding.root) {
        var mBinding = binding
    }
   interface ChangePwdListener{
       fun changeCode()
       fun clearMemory(context: Context,data: List<AppInfo>)
       fun clearAppData(context: Context)
   }
}