package com.xhly.manageapp.utils;

import java.text.SimpleDateFormat;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author ywx
 * @classname SystemClock
 * @description todo
 * @date 2022/11/9 20:31
 **/
public class SystemClock {
    private static final SystemClock MILLIS_CLOCK = new SystemClock(1);
    private final long precision;
    private final AtomicLong now;

    private SystemClock(long precision) {
        this.precision = precision;
        now = new AtomicLong(System.currentTimeMillis());
        scheduleClockUpdating();
    }

    public static SystemClock millisClock() {
        return MILLIS_CLOCK;
    }

    private void scheduleClockUpdating() {
        ScheduledExecutorService scheduler = Executors.newSingleThreadScheduledExecutor(runnable -> {
            Thread thread = new Thread(runnable, "system.clock");
            thread.setDaemon(true);
            return thread;
        });
        scheduler.scheduleAtFixedRate(() -> now.set(System.currentTimeMillis()), precision, precision, TimeUnit.MILLISECONDS);
    }

    public long now() {
        return now.get();
    }

    public static int getSecondTimestamp(){
        String timestamp = String.valueOf(SystemClock.millisClock().now()/1000);
        return Integer.valueOf(timestamp);
    }

    public static String getDateString(Long time){
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy年MM月d日 HH:mm:ss");
        return dateFormat.format(time);
    }
}
