package com.xhly.manageapp.workmanager

import android.app.usage.UsageEvents
import android.app.usage.UsageStatsManager
import android.content.Context
import androidx.appcompat.app.AppCompatActivity
import androidx.work.Worker
import androidx.work.WorkerParameters
import com.xhly.corelib.Const
import com.xhly.corelib.eventbus.UIEvent
import com.xhly.corelib.network.RetrofitService
import com.xhly.corelib.utils.AppUtils
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.SharedPreferencesUtils
import com.xhly.corelib.utils.toFormat
import com.xhly.manageapp.ManageApplication
import com.xhly.manageapp.bean.log.LogAppStartBean
import com.xhly.manageapp.bean.user.UserBean
import com.xhly.manageapp.network.log.LogService
import com.xhly.manageapp.service.websocket.AppSocket
import com.xhly.manageapp.ui.login.netservice.LoginService
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import java.util.Calendar

/**
 * 用来记录应用使用时长的定时任务
 */
class TimingWorker(context: Context, workerParams: WorkerParameters) :
    Worker(context, workerParams) {
    private val loginService by lazy { RetrofitService.create<LoginService>() }
    private val logService by lazy { RetrofitService.create<LogService>() }
    private val serviceScope = CoroutineScope(MainScope().coroutineContext)
    private val spUtils by lazy {
        SharedPreferencesUtils.getInstance(context)
    }

    companion object {
        //在activity中判断，如果连续10分钟都没执行则重启任务。
        var lastTime = 0L

        //上次策略请求时间
        private var StrategyTime = 0L
    }

    override fun doWork(): Result {
        val result: Result = Result.success()
        serviceScope.launch {
            LogShow("执行定时任务吧")
            var userBean: UserBean? = null
            try {
                val fromJson = spUtils.getFromJson(Const.USERINFO, UserBean().javaClass)
                userBean = fromJson as UserBean?
            } catch (e: Exception) {
                LogShow("问题是$e")
            }
            try {
                val appInfo = getAppInfo()
                if (appInfo.size > 0) {
                    logService.postLogAppStartAdd(appInfo)
                }
            }catch (e:Exception){
                LogShow("问题是$e")
            }
            UIEvent(Const.STARTLOCATION).post()
            userBean?.let {
               try {
                   val map = HashMap<String, Any>()
                   map["sn"] = ManageApplication.getDeviceInfo() ?: ""
                   val postCheckSn = loginService.postCheckSn(map)
                   if (postCheckSn.code == 0 && postCheckSn.obj) {
                       UIEvent(Const.ACCOUNTLOGINOUTSUCESS).post()
                   }
               }catch (e:Exception){
                 LogShow("问题是$e")
               }
            }
            //校验登录
            userBean?.let {
                val map = HashMap<String, Any>()
                map["regionCode"] = userBean.regionCode.toString()
                map["sn"] = ManageApplication.getDeviceInfo()?:""
                try {
                    val postLoginCheck = loginService.postLoginCheck(map)
                    LogShow("sn校验结果"+postLoginCheck.code+"|||"+postLoginCheck.obj.needLogin)
                    if (postLoginCheck.code == 0 && postLoginCheck.obj.needLogin) {
                        UIEvent(Const.ACCOUNTLOGINOUT).post()
                    }
                } catch (e: Exception) {
                    LogShow("解析异常")
                }
            }
            //创建socket
            userBean?.let {
                ManageApplication.instance?.let {
                    AppSocket.startSocket(it)
                }
            }
            //超出30分钟则请求策略
            if (StrategyTime == 0L) {
                StrategyTime = System.currentTimeMillis()
            } else if (System.currentTimeMillis() - StrategyTime >= 30 * 60 * 1000) {
                StrategyTime = System.currentTimeMillis()
                UIEvent(Const.CODE2004.toString()).post()
            }
        }
        return result
    }

    private fun getAppInfo(): ArrayList<LogAppStartBean> {
        //统计使用时长
        val dataList = AppUtils.GetAppList(applicationContext)
        val instance = Calendar.getInstance()
        instance.set(Calendar.MILLISECOND, 0)
        val currentTime = instance.timeInMillis
        LogShow("上次时间" + lastTime.toFormat() + "|||本次时间" + currentTime.toFormat())
        if (lastTime == 0L) {
            lastTime = currentTime
        } else {
            lastTime = currentTime
        }
        val timeInMillis = currentTime - 1000 * 60 * 5
        val usageManager =
            applicationContext.getSystemService(AppCompatActivity.USAGE_STATS_SERVICE) as UsageStatsManager
        val startList = arrayListOf<LogAppStartBean>()
        val sn = ManageApplication.getDeviceInfo()
        if (usageManager != null) {
            val endTime = System.currentTimeMillis()
            val startTime = timeInMillis
            //queryEvents查询使用时长，时间范围设置后，返回的数据时间范围和设置的一致，但需要自己判断时长
            val queryEvents = usageManager.queryEvents(startTime, endTime)
            val event = UsageEvents.Event()

            while (queryEvents.hasNextEvent()) {
                queryEvents.getNextEvent(event)
                when (event.eventType) {
                    UsageEvents.Event.ACTIVITY_RESUMED -> {
                        val appStartBean = LogAppStartBean()
                        val first =
                            dataList.firstOrNull { event.packageName.equals(it.packageName) }
                        first?.let {
                            //只上传不在这个集合中的应用
                            if (!Const.systemSetAppList.contains(first.packageName)){
                                sn?.let {
                                    appStartBean.sn = sn
                                }
                                appStartBean.appName = first.name
                                appStartBean.appPackage = first.packageName
                                appStartBean.starttime = event.timeStamp.toFormat()
                                appStartBean.ltype = 1
                                appStartBean.versionName = first.versionName
                                appStartBean.versionNum = first.versionNum
                                startList.add(appStartBean)
                            }
                        }
                    }

                    UsageEvents.Event.ACTIVITY_PAUSED -> {
                        val appStartBean = LogAppStartBean()
                        var first =
                            dataList.firstOrNull { event.packageName.equals(it.packageName) }
                        first?.let {
                            if (!Const.systemSetAppList.contains(first.packageName)){
                                sn?.let {
                                    appStartBean.sn = sn
                                }
                                appStartBean.appName = first.name
                                appStartBean.appPackage = first.packageName
                                appStartBean.starttime = event.timeStamp.toFormat()
                                appStartBean.ltype = 2
                                appStartBean.versionName = first.versionName
                                appStartBean.versionNum = first.versionNum
                                startList.add(appStartBean)
                            }
                        }
                    }
                }
            }
        }
        return startList
    }

    override fun onStopped() {
        super.onStopped()
        try {
            serviceScope.cancel()
        } catch (e: Exception) {

        }
    }
}